import org.apache.commons.codec.binary.Base64
import org.apache.commons.io.IOUtils
import org.apache.log4j.Logger


gliffy = new GliffyTool(restUserName: restUserName, restPassword: restPassword, baseurl: baseurl)

/**
 * Velocity tool to get Gliffy diagram images from the Gliffy REST API.
 */
class GliffyTool {
	def log = Logger.getLogger(this.getClass())

	/* Gliffy REST API request parameters. */
	String baseurl
	String restUserName
	String restPassword

	def getDiagramImage(issueKey, filename) {
		log.debug("Downloading Gliffy diagram image <${filename}> at ${issueKey}")
		HttpURLConnection connection
		InputStream inputStream
		ByteArrayOutputStream outputStream

		try {
			def fileTitle = URLEncoder.encode(filename.replaceAll("\\.gliffy", ""))
			def url = "${baseurl}/rest/gliffy/1.0/diagrams/fallback?issue=${issueKey}&title=${fileTitle}"
			log.debug("Gliffy URL for <${filename}> at ${issueKey}: ${url}")
			connection = new URL(url).openConnection()
			String authValue = "${restUserName}:${restPassword}"
			connection.setRequestProperty("Authorization", "Basic " + Base64.encodeBase64String(authValue.getBytes()))
			inputStream = connection.getInputStream()
			outputStream = new ByteArrayOutputStream()
			IOUtils.copy(inputStream, outputStream)
			log.debug("${outputStream.size()} bytes downloaded for <${filename}> at ${issueKey}")

			String image = Base64.encodeBase64String(outputStream.toByteArray())
			return "data:image/png;base64,${image}"
		} catch (Throwable e) {
			log.error("Failed to download Gliff diagram image", e)
			return e
		} finally {
			connection.disconnect()
			IOUtils.closeQuietly(inputStream)
			IOUtils.closeQuietly(outputStream)
		}
	}
}
